//+------------------------------------------------------------------+
//|                                                        json1.mqh |
//| Parse a JSON String                                              |
//| (c) 2014 Andrew Lord forex@NICKNAME@lordy.org.uk                 |
//|                                    Copyright (c) 2018, Marketeer |
//|                          https://www.mql5.com/en/users/marketeer |
//+------------------------------------------------------------------+


#include <hash1.mqh>

#define Object JSONObject
#define Array JSONArray

/// Different types of JSON Values
enum ENUM_JSON_TYPE { JSON_NULL, JSON_OBJECT, JSON_ARRAY, JSON_NUMBER, JSON_STRING, JSON_BOOL, JSON_INTEGER };


/// Proxy object for passing parameters into getter methods, used for return type resolution
template<typename T>
class JSONGetter
{
  private:
    string key;
    int _index;
    
  public:
    JSONGetter() {}
    JSONGetter *set(const string k) { key = k; return &this; }
    JSONGetter *set(const int i) { _index = i; return &this; }
    string get() const { return key; }
    int index() const { return _index; }
};


JSONGetter<string> _StringGetter;
JSONGetter<double> _DoubleGetter;
JSONGetter<long> _LongGetter;
JSONGetter<bool> _BoolGetter;

/// Proxy object for passing parameters into setter methods, used for return type resolution
template<typename T>
class JSONSetter
{
  private:
    string key;
    int _index;
    
  public:
    JSONSetter() {}
    JSONSetter *set(const string k) { key = k; return &this; }
    JSONSetter *set(const int i) { _index = i; return &this; }
    string get() const { return key; }
    int index() const { return _index; }
};

// this objects help to add new empty variables/properties of plain types into Objects
JSONSetter<string> _StringSetter;
JSONSetter<double> _DoubleSetter;
JSONSetter<long> _LongSetter;
JSONSetter<bool> _BoolSetter;

#define _S(K) (_StringGetter.set(K))
#define _D(K) (_DoubleGetter.set(K))
#define _L(K) (_LongGetter.set(K))
#define _B(K) (_BoolGetter.set(K))

#define S_(K) (_StringSetter.set(K))
#define D_(K) (_DoubleSetter.set(K))
#define L_(K) (_LongSetter.set(K))
#define B_(K) (_BoolSetter.set(K))

#define __O(S) (new JSONObject((S), JSON_FLAG_REPLACE_SINGLETICKS))
#define __A(S) (new JSONArray((S), JSON_FLAG_REPLACE_SINGLETICKS))

#define JSON_FLAG_REPLACE_BACKTICKS   1
#define JSON_FLAG_REPLACE_SINGLETICKS 2
#define JSON_FLAG_MOVE_OBJECTS        4

//forward declarations
class JSONObject;
class JSONArray;
class JSONNull;


///
/// Generic class for all JSON types (Number, String, Bool, Array, Object )
/// It is a subclass of HashValue so it can be stored in an JSONObject hash
///
class JSONValue : public HashValue
{
  private:
    ENUM_JSON_TYPE _type;
    
  protected:
    static int _flags; // global flags
    static int _digits;
    bool empty;
    bool error;
    
    template<typename T>
    void initializer(T v)
    {
      T x;
      if(is<T>())
      {
        x = get<T>();
      }

      bool b = JSONValue::get<T>(&this, x);

      JSONGetter<T> ptr;
      this[&ptr];
      
      this = v;

      T array[];
      extend<T>(array);
    }

  public:
    JSONValue(): empty(true), error(false) {}
    ~JSONValue() {}
    
    static void setFlags(const int flag)
    {
      _flags = flag;
    }
    
    static int getFlags()
    {
      return _flags;
    }

    static void setDigits(const int precision)
    {
      _digits = precision;
    }

    ENUM_JSON_TYPE getType()
    {
      return _type;
    }
    
    void setType(ENUM_JSON_TYPE t)
    {
      _type = t;
    }

    /// True if JSONValue is a instance of JSONNull
    bool isNull()
    {
      return _type == JSON_NULL;
    }

    /// True if JSONValue is a instance of JSONObject
    bool isObject()
    {
      return _type == JSON_OBJECT;
    }

    /// True if JSONValue is a instance of JSONArray
    bool isArray()
    {
      return _type == JSON_ARRAY;
    }

    bool isBaseType()
    {
      return !(isNull() || isObject() || isArray());
    }
    
    double getNumber()
    {
      switch(_type)
      {
        case JSON_BOOL: return get<bool>() ? 1.0 : 0.0;
        case JSON_INTEGER: return (double)get<long>();
        case JSON_NUMBER: return get<double>();
        case JSON_NULL: return 0.0;
        case JSON_STRING: return StringToDouble(get<string>());
        case JSON_ARRAY:
        case JSON_OBJECT:
          return DBL_MAX;
      }
      return 0;
    }

    // Override in child classes
    virtual string toString(bool raw = false)
    {
      return "";
    }
    
    virtual bool isEmpty()
    {
      return empty;
    }
    
    bool isError()
    {
      return error;
    }

    /// If this JSONValue is an instance of base type return it (or cast will fail)
    template<typename T>
    T get()
    {
      return dynamic_cast<JSONBaseValue<T> *>(&this).get();
    }

    /// True if JSONValue is an instance of specified base type
    template<typename T>
    bool is()
    {
      return dynamic_cast<JSONBaseValue<T> *>(&this) != NULL;
    }


    /// Get value of the JSONValue, without Program termination
    /// @param val : Base type object from which value will be extracted.
    /// @param out : The extracted value.
    /// @return true if OK else false
    template<typename T>
    static bool get(JSONValue *val, T &out)
    {
      if(val != NULL && val.is<T>())
      {
        out = val.get<T>();
        return true;
      }
      return false;
    }

    template<typename T>
    T operator[](JSONGetter<T> *ptr)
    {
      JSONArray *array = dynamic_cast<JSONArray *>(&this); // templates can not be virtual, so doing manual type resolution
      if(array != NULL)
      {
        JSONValue *value = array.getValue(ptr.index());
        if(value == NULL) return NULL;
        if(!value.is<T>())
        {
          Print("Can't read index ", ptr.index(), " in Array: ", typename(T), " asked, got ", EnumToString(value.getType()));
        }
        return value.get<T>();
      }
      JSONObject *object = dynamic_cast<JSONObject *>(&this);
      if(object != NULL)
      {
        JSONValue *value = object.getValue(ptr.get());
        if(value == NULL) return NULL;
        if(!value.is<T>())
        {
          Print("Can't read key ", ptr.get(), " in Object: ", typename(T), " asked, got ", EnumToString(value.getType()));
        }
        return value.get<T>();
      }
      return NULL;
    }

    // object/array specialization
    template<typename T>
    void operator=(T *v)
    {
      JSONObject *object = dynamic_cast<JSONObject *>(&this);
      if(object != NULL)
      {
        object.reset();
        if((_flags & JSON_FLAG_MOVE_OBJECTS) != 0)
        {
          object.extend(v); // this will effectively move values of properties from v to this
          if(CheckPointer(v) == POINTER_DYNAMIC) delete v;
        }
        else
        {
          object.clone(v);
        }
      }
      else
      {
        this.clone(v);
      }
    }

    // plain data types specialization
    template<typename T>
    void operator=(T v)
    {
      if(typename(T) == "int")
      {
        Print("Don't use `int`! Use `long` instead!");
      }
      JSONBaseValue<T> *base = dynamic_cast<JSONBaseValue<T> *>(&this);
      if(base != NULL) base.set(v);
    }
    
    virtual void extend(JSONValue *object) = 0;
    
    virtual void expand(string json)
    {
      extend(json, JSONValue::getFlags());
    }
    
    virtual void extend(string json, int flags = 0)
    {
      error = false;
      JSONParser parser;
      JSONObject *object = dynamic_cast<JSONObject *>(parser.parse(json, flags));
      if(object != NULL)
      {
        extend(object);
        delete object;
      }
      else
      {
        Print("Failed to create Object from JSON: "/*, json*/);
        Print("===");
        if(parser.getErrorCode())
        {
          Print(parser.getErrorCode(), " ", parser.getErrorMessage());
          Print("---");
          Print(parser.getPosition(), " ", StringSubstr(json, parser.getPosition(), 32));
        }
        else
        {
          Print("Not a JSON");
        }
        error = true;
      }
    }

    template<typename T>
    void extend(T &array[])
    {
      if(_type == JSON_ARRAY)
      {
        string s = "";
        for(int i = 0; i < ArraySize(array); i++)
        {
          s += (i > 0 ? "," : "") + (string)array[i];
        }
        extend("[" + s + "]");
      }
      else
      {
        Print("Object of type ", EnumToString(_type), " ", typename(this) , " can't be extended with array of ", typename(T));
      }
    }
    
    virtual JSONValue *clone(JSONValue *object)
    {
      _type = object._type;
      return &this;
    }

    virtual JSONValue *operator[](string key) = 0;
    virtual JSONValue *operator[](int index) = 0;
    
    virtual JSONValue *clone2() = 0;

    /// Lookup key and get associated array, NULL if not present. Cast failure if not an Array.
    JSONArray *getArray(string key)
    {
      return dynamic_cast<JSONArray *>(getValue(key));
    }
    
    /// Lookup key and get associated Object, NULL if not present. Cast failure if not an Object.
    JSONObject *getObject(string key)
    {
      return dynamic_cast<JSONObject *>(getValue(key));
    }

    JSONObject *operator<<(string key)
    {
      return getObject(key);
    }
  
    // sugar
    virtual JSONValue *getValue(string key)
    {
      if(key == NULL) return &this;
      return NULL;
    }
};

// -----------------------------------------

template<typename T>
class JSONBaseValue : public JSONValue
{
  private:
    T _v;

    void deduceType()
    {
      if(typename(T) == "string") setType(JSON_STRING);
      else if(typename(T) == "double" || typename(T) == "float") setType(JSON_NUMBER);
      else if(typename(T) == "int" || typename(T) == "long") setType(JSON_INTEGER);
      else if(typename(T) == "bool") setType(JSON_BOOL);
    }

  public:
    JSONBaseValue(T s)
    {
      set(s);
      deduceType();
    }

    JSONBaseValue()
    {
      deduceType();
    }
    
    T get()
    {
      return _v;
    }
    
    void set(T v)
    {
      _v = v;
      empty = false;
    }
    
    virtual JSONValue *clone(JSONValue *object) override
    {
      JSONBaseValue<T> *base = dynamic_cast<JSONBaseValue<T> *>(object);
      if(base != NULL && base.getType() == getType()) // we can't change object type T
      {
        _v = base.get();
        empty = false;
      }
      return &this;
    }

    // specialization for base data type wrapper classes
    void operator=(JSONBaseValue<T> *v)
    {
      if(v != NULL && getType() == v.getType())
      {
        clone(v);
      }
      //else
      //{
      //  Print("Incompatible types ", EnumToString(getType()), " and ", EnumToString(v.getType()));
      //}
    }

    virtual string toString(bool raw = false) override
    {
      if(getType() == JSON_STRING)
      {
        if(!raw) return "\"" + (string)_v + "\"";
      }
      else
      if(getType() == JSON_NUMBER)
      {
        return StringFormat("%" + (string)JSONValue::_digits + "f", _v);
      }
      return (string)_v;
    }
    
    virtual void extend(JSONValue *object) override
    {
    }
    
    virtual JSONValue *operator[](string key) override
    {
      return NULL;
    }
    
    virtual JSONValue *operator[](int index) override
    {
      return NULL;
    }
    
    virtual JSONValue *clone2() override
    {
      return (new JSONBaseValue<T>()).clone(&this);
    }
};

/// This class should not be necessary, but null is genrally infrequent so
/// I havent bothered to code it away yet.
class JSONNull : public JSONValue
{
  public:
    JSONNull()
    {
      setType(JSON_NULL);
    }
    ~JSONNull() {}

    virtual string toString(bool raw = false) override
    {
      return "null";
    }
    
    virtual void extend(JSONValue *object) override
    {
    }
    
    virtual JSONValue *operator[](string key) override
    {
      return NULL;
    }
    
    virtual JSONValue *operator[](int index) override
    {

      return NULL;
    }
    virtual JSONNull *clone2() override
    {
      return (new JSONNull()).clone(&this);
    }
};


/// This represents a JSONObject which is represented internally as a Hash
class JSONObject : public JSONValue
{
  protected:
    Hash *_hash;
    
    template<typename T>
    void initializer2(T v)
    {
      JSONSetter<T> setter;
      JSONBaseValue<T> *ptr = this[setter.set("key")];
      ptr = v;

      T x = get<T>("key");
    
      T y;
      bool b = get<T>("key", y);
    
      x = safeRead<T>("key", v);
    }

  public:
    JSONObject()
    {
      setType(JSON_OBJECT);
    }

    JSONObject(string json, int flags = 0)
    {
      setType(JSON_OBJECT);
      // WARNING! this is a virtual method, but it's called from constructor,
      // only JSONObject's is invoked
      extend(json, flags);
    }
    
    ~JSONObject()
    {
      reset();
    }
    
    virtual void reset()
    {
      if(_hash != NULL)
      {
        delete _hash;
        _hash = NULL;
      }
      empty = true;
    }

    /// Lookup key and get associated value - halt program if wrong type(cast error) or doesnt exist(null pointer)
    template<typename T>
    T get(string key)
    {
      return getValue(key).get<T>();
    }

    /// Lookup key and get associated value of specified type, return false on failure.
    template<typename T>
    bool get(string key, T &out)
    {
      return get<T>(getValue(key), out);
    }

    /// Lookup key and get associated value - best for data whose structure might change as any type can safely be returned.
    virtual JSONValue *getValue(string key) override
    {
      if(_hash == NULL) return NULL;
      return (JSONValue *)_hash.get(key);
    }
    
    virtual JSONValue *operator[](string key) override
    {
      return getValue(key);
    }
    
    virtual JSONValue *operator[](int index) override
    {
      return NULL; // no value
    }

    template<typename T>
    JSONBaseValue<T> *operator[](JSONSetter<T> *ptr)
    {
      JSONBaseValue<T> *value = new JSONBaseValue<T>();
      put(ptr.get(), value);
      return value;
    }
    
    /// Store an object value against the specified key string - Used by the parser.
    void put(string key, JSONValue *v)
    {
      if(_hash == NULL) _hash = new Hash();
      HashValue *old = _hash.put(key, v);
      if(CheckPointer(old) == POINTER_DYNAMIC) delete old;
      empty = false;
    }
    
    JSONValue *move(string key)
    {
      if(_hash == NULL) _hash = new Hash();
      return _hash.put(key, NULL);
    }
    
    void remove(string key)
    {
      if(_hash != NULL)
      {
        _hash.del(key);
      }
    }

    virtual string toString(bool raw = false) override
    {
      string s = "{";
      if(_hash != NULL)
      {
        HashLoop *l;
        int n = 0;

        for(l = new HashLoop(_hash); l.hasNext(); l.next())
        {
          JSONValue *v = (JSONValue *)(l.val());
          s = s + (++n == 1 ? "" : ",") + "\"" + l.key() + "\":" + (v != NULL ? v.toString() : "null");
        }
        delete l;
      }
      s = s + "}";
      return s;
    }

    /// Return the internal Hash - Used by JSONIterator
    Hash *getHash()
    {
      return _hash;
    }

    bool containsKey(const string key)
    {
      return (_hash != NULL && _hash.containsKey(key));
    }
    
    int keys(string &result[])
    {
      if(_hash == NULL) return 0;
      
      JSONIterator it(&this);
      ArrayResize(result, _hash.getCount());

      for(int i = 0; it.hasNext(); it.next(), i++)
      {
        result[i] = it.key();
      }
      return ArraySize(result);
    }

    virtual void extend(JSONValue *v) override
    {
      if(v == NULL) return; // support optional extensions without if-s
      
      JSONObject *object = dynamic_cast<JSONObject *>(v);
      if(object == NULL)
      {
        Print("Object.extend failed, ", EnumToString(v.getType()));
        return;
      }
      
      if(object.getType() != JSON_OBJECT)
      {
        Print("Can't extend Object from ", EnumToString(object.getType()));
        return;
      }
      
      if(v == &this) return;
      
      
      JSONIterator it(object);

      for(; it.hasNext(); it.next())
      {
        string k = it.key();
        JSONValue *w = it.val();
        if(!containsKey(it.key()) || (it.val() != NULL && !it.val().isObject() && !it.val().isArray()))
        {
          if((_flags & JSON_FLAG_MOVE_OBJECTS) != 0)
            put(it.key(), object.move(it.key())); // add or replace a flat value
          else
            put(it.key(), it.val().clone2());
        }
        else
        {
          if(getValue(it.key()) == NULL)
          {
            Print("this[", it.key(), "]=NULL");
            if((_flags & JSON_FLAG_MOVE_OBJECTS) != 0)
              put(it.key(), object.move(it.key())); // this modifies source as well!
            else
              put(it.key(), it.val().clone2());
          }
          else
          {
            if(it.val() == NULL)
            {
              Print("other[", it.key(), "]=NULL");
              put(it.key(), NULL);
            }
            else
            {
              if(getValue(it.key()).getType() != it.val().getType())
              {
                Print("Note on extending `", it.key(), "`: was ", EnumToString(getValue(it.key()).getType()), " now ", EnumToString(it.val().getType()));
              }
              
              if(containsKey(it.key()) && getValue(it.key()).isObject())
              {
                ((JSONObject *)getValue(it.key())).extend((JSONObject *)it.val()); // merge properties, object can only be extended with objects
              }
              else if(containsKey(it.key()) && getValue(it.key()).isArray())
              {
                ((JSONArray *)getValue(it.key())).extend((JSONArray *)it.val()); // add elements, arrays can only be extended with arrays
              }
              else // (contains key and new is not object) OR (does not contain key and new is object [not important, move as a whole])
              {
                if((_flags & JSON_FLAG_MOVE_OBJECTS) != 0)
                  put(it.key(), object.move(it.key())); // this modifies source as well!
                else
                  put(it.key(), it.val().clone2());
              }
            }
          }
        }
      }
    }

    virtual JSONValue *clone(JSONValue *v) override
    {
      JSONObject *object = dynamic_cast<JSONObject *>(v);
      if(object != NULL && object.getType() == getType())
      {
        reset();
        
        JSONIterator it(object);
        for(; it.hasNext(); it.next())
        {
          JSONValue *complex = dynamic_cast<JSONObject *>(it.val());
          if(complex == NULL)
          {
            if(it.val() == NULL)
            {
              Print("NULL @ ", it.key(), " in ", object.toString());
            }

            switch(it.val().getType())
            {
              case JSON_NULL:
                put(it.key(), new JSONNull());
                break;
              case JSON_NUMBER:
                put(it.key(), new JSONBaseValue<double>(it.val().get<double>()));
                break;
              case JSON_STRING:
                put(it.key(), new JSONBaseValue<string>(it.val().get<string>()));
                break;
              case JSON_BOOL:
                put(it.key(), new JSONBaseValue<bool>(it.val().get<bool>()));
                break;
              case JSON_INTEGER:
                put(it.key(), new JSONBaseValue<long>(it.val().get<long>()));
                break;
              default:
                Print("Can't clone type ", EnumToString(it.val().getType()));
            }
          }
          else
          {
            if(it.val().isObject())
            {
              put(it.key(), it.val().clone2());
            }
            else
            if(it.val().isArray())
            {
              put(it.key(), it.val().clone2());
            }
          }
        }
      }
      return &this;
    }

    virtual JSONObject *clone2() override
    {
      return (new JSONObject()).clone(&this);
    }

    template<typename T>
    T safeRead(const string key, T defaultValue)
    {
      if(containsKey(key) && CheckPointer(getValue(key)) != POINTER_INVALID && getValue(key).is<T>())
      {
        return getValue(key).get<T>();
      }
      return defaultValue;
    }

    static Object *indexArrayBy(Array *array, string key)
    {
      if(array == NULL) return NULL;

      Object *result = __O("{}");
      for(int i = 0; i < array.size(); i++)
      {
        if(array[i].isObject() && ((Object *)array[i]).containsKey(key))
        {
          result.put(array[i][_S(key)], array[i]);
        }
      }
      return result;
    }

    static Object *indexBy(Object *object, string key)
    {
      if(object == NULL) return NULL;

      Array *array = dynamic_cast<Array *>(object);
      if(array != NULL) return indexArrayBy(array, key);
    
      Object *result = __O("{}");
      
      JSONIterator it(object);
      for(; it.hasNext(); it.next())
      {
        Object *element = (Object *)it.val();
        if(element != NULL && element.containsKey(key))
        {
          result.put(element[_S(key)], element);
        }
      }
      
      return result;
    }

};


template<typename T>
class COMPARE
{
  public:
    virtual int Compare(T *First, T *Second) const = 0;
};


template<typename K,typename A>
class CompareAll : public COMPARE<A>
{
  private:
    K key;
    bool numeric;
    bool ascending;
    
  public:
    CompareAll(const K k, bool n = true, bool a = true): key(k), numeric(n), ascending(a) {}
    
    int Compare(A *First, A *Second) const override
    {
      JSONValue *v1 = First.getValue(key);
      JSONValue *v2 = Second.getValue(key);
      if(numeric)
      {
        double delta = (v1 != NULL ? v1.getNumber() : 0) - (v2 != NULL ? v2.getNumber() : 0);
        return (delta > 0 ? +1 : (delta < 0 ? -1 : 0)) * (ascending ? +1 : -1);
      }
      else
      {
        return StringCompare((v1 != NULL ? v1.toString(true) : ""), (v2 != NULL ? v2.toString(true) : "")) * (ascending ? +1 : -1);
      }
    }
};


/// This is a JSONArray which is represented internally as a MQL4 dynamic array of JSONValue *
class JSONArray : public JSONObject // JSONValue
{
  private:
    int _size;
    JSONValue *_array[];
    
 protected:
    
    template<typename T>
    void initializer3(T v)
    {
      this <= v;

      T x = get<T>(_size - 1);
    
      bool b = get<T>(0, x);

      CompareAll<int,Array> cmp1(0, true, true);
      CompareAll<string,Object> cmp2("", true, true);
    
      QuickSort(cmp1, 0, 0);
      QuickSort(cmp2, 0, 0);
    }
    
    
  public:
    JSONArray(): _size(0)
    {
      setType(JSON_ARRAY);
    }
    
    JSONArray(const string json, int flags = 0): _size(0)
    {
      setType(JSON_ARRAY);
      extend(json, flags);
    }
    
    ~JSONArray()
    {
      reset();
    }
    
    virtual void reset() override
    {
      // clean up array
      for(int i = ArrayRange(_array, 0) - 1; i >= 0; i--)
      {
        if(CheckPointer(_array[i]) == POINTER_DYNAMIC) delete _array[i];
      }
      _size = 0;
      ArrayResize(_array, 0);
      empty = true;
    }
    
    // Getters for Objects (key lookup ) --------------------------------------

    /// Lookup value by array index - halt program if wrong type(cast error) or doesnt exist(null pointer)
    template<typename T>
    T get(int index)
    {
      return getValue(index).get<T>();
    }
    
    /// Lookup a base type value by array index. NULL if not present. Cast failure if not an Object.
    template<typename T>
    bool get(int index, T &out)
    {
      return get<T>(getValue(index), out);
    }

    /// Lookup array child by index, NULL if not present. Cast failure if not an Array.
    JSONArray *getArray(int index)
    {
      return getValue(index);
    }

    /// Lookup object child by index, NULL if not present. Cast failure if not an Array.
    JSONObject *getObject(int index)
    {
      return getValue(index);
    }
    
    /// The following method allows any type to be returned. Use this when parsing unpredictable data
    JSONValue *getValue(int index)
    {
      if(index >= 0 && index < _size) return _array[index];
      return NULL;
    }
    
    virtual JSONValue *operator[](int index) override
    {
      return getValue(index);
    }

    virtual JSONValue *operator[](string key) override
    {
      return NULL;
    }

    JSONValue *moveValue(int index)
    {
      JSONValue *result = _array[index];
      _array[index] = NULL;
      return result;
    }

    /// Used by the Parser when building the array
    bool put(int index, JSONValue *v)
    {
      if(index >= _size)
      {
        int oldSize = _size;
        int newSize = ArrayResize(_array, index + 1, 30);
        if(newSize <= index) return false;
        _size = newSize;

        // initialise
        for(int i = oldSize ; i < newSize ; i++) _array[i] = NULL;
      }
      // Delete old entry if any
      if(_array[index] != NULL) delete _array[index];

      //set new entry
      _array[index] = v;
      empty = false;

      return true;
    }

    bool push(JSONValue *v)
    {
      return put(_size, v);
    }
    
    JSONArray *operator<<(JSONValue *v)
    {
      push(v);
      return &this;
    }

    template<typename T>
    JSONArray *operator<=(T v)
    {
      push(new JSONBaseValue<T>(v));
      return &this;
    }
    
    void swap(const int i, const int j)
    {
      JSONValue *temp = _array[i];
      _array[i] = _array[j];
      _array[j] = temp;
    }

    virtual string toString(bool raw = false) override
    {
      string s = "[";
      if(_size > 0)
      {
        s = s + _array[0].toString();
        for(int i = 1 ; i < _size ; i++)
        {
          s = s + "," + _array[i].toString();
        }
      }
      s = s + "]";
      return s;
    }

    int size()
    {
      return _size;
    }

    virtual void extend(JSONValue *v) override
    {
      if(v == NULL) return;
      
      JSONArray *array = dynamic_cast<JSONArray *>(v);
      if(array == NULL)
      {
        Print("Array.extend failed, ", (v != NULL ? EnumToString(v.getType()) : "NULL"));
        return;
      }
      
      if(v == &this) return;
      
      for(int i = 0; i < array.size(); i++)
      {
        put(_size, array.moveValue(i));
      }
    }
    
    virtual JSONValue *clone(JSONValue *v) override
    {
      JSONArray *object = dynamic_cast<JSONArray *>(v);
      if(object != NULL && object.getType() == getType())
      {
        reset();
        
        JSONIterator it(object);
        for(int i = 0; it.hasNext(); it.next(), i++)
        {
          JSONValue *complex = dynamic_cast<JSONObject *>(it.val());
          if(complex == NULL)
          {
            switch(it.val().getType())
            {
              case JSON_NULL:
                put(i, new JSONNull());
                break;
              case JSON_NUMBER:
                put(i, new JSONBaseValue<double>(it.val().get<double>()));
                break;
              case JSON_STRING:
                put(i, new JSONBaseValue<string>(it.val().get<string>()));
                break;
              case JSON_BOOL:
                put(i, new JSONBaseValue<bool>(it.val().get<bool>()));
                break;
              case JSON_INTEGER:
                put(i, new JSONBaseValue<long>(it.val().get<long>()));
                break;
              default:
                Print("Can't clone type ", EnumToString(it.val().getType()));
            }
          }
          else
          {
            if(it.val().isObject())
            {
              put(i, it.val().clone2());
            }
            else
            if(it.val().isArray())
            {
              put(i, it.val().clone2());
            }
          }
        }
      }
      return &this;
    }
    
    virtual JSONArray *clone2() override
    {
      return (new JSONArray()).clone(&this);
    }

    template<typename T>
    static Array *filterByArray(Array *data, string field, Array *filter)
    {
      if(filter == NULL || field == NULL) return data;
      
      Array *result = new Array();
      for(int i = 0; i < data.size(); i++)
      {
        Object *entry = data[i];
        if(entry != NULL)
        {
          T value = entry.safeRead<T>(field, NULL);
          for(int j = 0; j < filter.size(); j++)
          {
            JSONValue *f = filter[j];
            if(f.is<T>() && (f.get<T>() == value))
            {
              result.push(data.moveValue(i));
              break; // by j
            }
          }
        }
        
      }
      delete data;
      return result;
    }

    // SORTING
    
    JSONArray *sort(const int column, bool ascending = true, bool numeric = true, int Count = WHOLE_ARRAY, const int Start = 0)
    {
      if(Count == WHOLE_ARRAY)
        Count = this.size();
  
      CompareAll<int,Array> cmp0(column, numeric, ascending);
      QuickSort(cmp0, Start, Start + Count - 1);
      return &this;
    }

    JSONArray *sort(const string field, bool ascending = true, bool numeric = true, int Count = WHOLE_ARRAY, const int Start = 0)
    {
      if(Count == WHOLE_ARRAY)
        Count = this.size();
  
      CompareAll<string,Object> cmp0(field, numeric, ascending);
      QuickSort(cmp0, Start, Start + Count - 1);
      return &this;
    }


  private:
  
    template<typename T2>
    void QuickSort(const T2 &Compare, const int Start, const int End)
    {
      if(Start < End)
      {
        const int Pivot = Partition(Compare, Start, End);
  
        QuickSort(Compare, Start, Pivot - 1);
        QuickSort(Compare, Pivot + 1, End);
      }
    }

    template<typename T2>
    int Partition(const T2 &Compare, const int Start, const int End)
    {
      int Marker = Start;
  
      for(int i = Start; i <= End && !IsStopped(); i++)
      {
        if(Compare.Compare(_array[i], _array[End]) <= 0)
        {
          swap(i, Marker);
  
          Marker++;
        }
      }
  
      return(Marker - 1);
    }

};



/// Parse JSON text using a simple recursive descent parser
/// Exmaple
///
/// <pre>
///    string s = "{ \"firstName\": \"John\","+
///       " \"lastName\": \"Smith\","+
///       " \"age\": 25,"+
///       " \"address\": { \"streetAddress\": \"21 2nd Street\", \"city\": \"New York\", \"state\": \"NY\", \"postalCode\": \"10021\" },"+
///       " \"phoneNumber\": [ { \"type\": \"home\", \"number\": \"212 555-1234\" }, { \"type\": \"fax\", \"number\": \"646 555-4567\" } ],"+
///       " \"gender\":{ \"type\":\"male\" }  }";
///
///    JSONParser *parser = new JSONParser();
///
///    JSONValue *jv = parser.parse(s);
///
///    if (jv == NULL) {

///
///        Print("error:"+(string)parser.getErrorCode()+parser.getErrorMessage());
///
///    } else {

///
///        Print("PARSED:"+jv.toString());
///
///        if (jv.isObject()) {

///
///            JSONObject *jo = jv;
///
///            // Direct access - will throw null pointer if wrong getter used.
///
///            Print("firstName:" + jo.getString("firstName"));
///            Print("city:" + jo.getObject("address").getString("city"));
///            Print("phone:" + jo.getArray("phoneNumber").getObject(0).getString("number"));
///
///            // Safe access in case JSON data is missing or different.
///
///            if (jo.getString("firstName",s) ) Print("firstName = "+s);
///
///            // Loop over object returning JSONValue
///
///            JSONIterator *it = new JSONIterator(jo);
///            for( ; it.hasNext() ; it.next()) {

///                Print("loop:"+it.key()+" = "+it.val().toString());
///            }
///            delete it;
///        }
///        delete jv;
///    }
///    delete parser;
/// </pre>

class JSONParser
{
  private:
    /// Current parse position
    int _pos;
    /// The input string is expanded into an array of ushort (wchar)
    ushort _in[];
    /// Length of string
    int _len;
    /// The original input string
    string _instr;

    int _errCode;
    string _errMsg;

    void setError(int code = 1, string msg = "unknown error")
    {
      _errCode |= code;
      if(_errMsg == "")
      {
        _errMsg = /*"JSONParser::Error: " + */msg;
      }
      else
      {
        _errMsg = _errMsg + "\n" + msg;
      }
      
      //string context = StringSubstr(_instr, MathMin(0, _pos - 10), 20);
      string before = StringSubstr(_instr, MathMax(0, _pos - 20), MathMin(20, _pos));
      string after = StringSubstr(_instr, _pos, 20);
      _errMsg += "\n" + before + "^" + after;
      
    }

    /// Parse a JSON Object
    JSONObject *parseObject()
    {
      JSONObject *o = new JSONObject();
      skipSpace();
      if(expect('{'))
      {
        while(_errCode == 0)
        {
          skipSpace();
          if(_in[_pos] != '"') break;

          // Read the key
          string key = parseString();

          if(_errCode != 0 || key == NULL) break;

          skipSpace();

          if(!expect(':')) break;

          // read the value
          JSONValue *v = parseValue();
          if(_errCode != 0) break;

          o.put(key, v);

          skipSpace();

          if(!expectOptional(',')) break;
        }
        if(!expect('}'))
        {
          setError(2, "Object: expected `\"` or `}` ");
        }
      }
      if(_errCode != 0)
      {
        delete o;
        o = NULL;
      }
      return o;
    }

    bool isDigit(ushort c)
    {
      return (c >= '0' && c <= '9') || c == '+'  || c == '-'  ;
    }

    bool isDoubleDigit(ushort c)
    {
      return (c >= '0' && c <= '9') || c == '+'  || c == '-'  || c == '.'  || c == 'e'  || c == 'E' ;
    }

    void skipSpace()
    {
      while(_in[_pos] == ' ' || _in[_pos] == '\t' || _in[_pos] == '\r' || _in[_pos] == '\n')
      {
        if(_pos >= _len) break;
        _pos++;
      }
    }

    bool expect(ushort c)
    {
      bool ret = false;
      if(c == _in[_pos])
      {
        _pos++;
        ret = true;
      }
      else
      {
        setError(1, ("Expected `" +
                     ShortToString(c) + "`(" + (string)c + ")" +
                     " got `" + ShortToString(_in[_pos]) + "`(" + (string)_in[_pos] + ")"));
      }
      return ret;
    }

    bool expectOptional(ushort c)
    {
      bool ret = false;
      if(c == _in[_pos])
      {
        _pos++;
        ret = true;
      }
      return ret;
    }

    ushort peek()
    {
      return _in[_pos];
    }

    string parseString()
    {
      string ret = "";
      if(expect('"'))
      {
        while(true)
        {
          int end = _pos;
          while(end < _len && _in[end] != '"' && _in[end] != '\\')
          {
            end++;
          }

          if(end >= _len)
          {
            setError(2, "missing quote: end" + (string)end + ":len" + (string)_len + ":" + ShortToString(_in[_pos]) + ":" + StringSubstr(_instr, _pos, 10) + "...");
            break;
          }
          // Check if character was escaped.
          // TODO \" \\ \/ \b \f \n \r \t \u0000
          if(_in[end] == '\\')
          {
            // Add partial string and get more
            ret = ret + StringSubstr(_instr, _pos, end - _pos);
            end++;
            if(end >= _len)
            {
              setError(4, "parse error after escape");
            }
            else
            {
              ushort c = 0;
              switch(_in[end])
              {
                case '"':
                case '\\':
                case '/':
                  c = _in[end];
                  break;
                case 'b':
                  c = 8;
                  break; // backspace - 8
                case 'f':
                  c = 12;
                  break; // form feed 12
                case 'n':
                  c = '\n';
                  break;
                case 'r':
                  c = '\r';
                  break;
                case 't':
                  c = '\t';
                  break;
                default:
                  setError(3, "unknown escape");
              }
              if(c == 0) break;
              ret = ret + ShortToString(c);
              _pos = end + 1;
            }
          }
          else if(_in[end] == '"')
          {
            // End of string
            ret = ret + StringSubstr(_instr, _pos, end - _pos);
            _pos = end + 1;
            break;
          }
        }
      }
      if(_errCode != 0)
      {
        ret = NULL;
      }
      return ret;
    }

    JSONValue *parseValue(ushort gracefulchar = 0)
    {
      JSONValue *ret = NULL;
      skipSpace();

      if(_in[_pos] == '[')
      {

        ret = (JSONValue*)parseArray();

      }
      else if(_in[_pos] == '{')
      {

        ret = (JSONValue*)parseObject();

      }
      else if(_in[_pos] == '"')
      {

        string s = parseString();
        ret = (JSONValue*)new JSONBaseValue<string>(s);

      }
      else if(isDoubleDigit(_in[_pos]))
      {

        bool isDoubleOnly = false;
        long l = 0;
        long sign;
        // number
        int i = _pos;

        if(_in[_pos] == '-')
        {
          sign = -1;
          _pos++;
          i++;
        }
        else if(_in[_pos] == '+')
        {
          sign = 1;
          _pos++;
          i++;
        }
        else
        {
          sign = 1;
        }

        while(i < _len && isDigit(_in[i]))
        {
          l = l * 10 + (_in[i] - '0');
          i++;
        }
        if(isDoubleDigit(_in[i]))
        {
          // Looks like a real number;
          while(i < _len && isDoubleDigit(_in[i]))
          {
            i++;
          }
          string s = StringSubstr(_instr, _pos, i - _pos);
          double d = sign * StringToDouble(s);
          ret = (JSONValue*)new JSONBaseValue<double>(d); // Create a Number as double
        }
        else
        {
          l = sign * l;
          ret = (JSONValue*)new JSONBaseValue<long>(l); // Create a Number as a long
        }
        _pos = i;

      }
      else if(_in[_pos] == 't' && StringSubstr(_instr, _pos, 4) == "true")
      {

        ret = (JSONValue*)new JSONBaseValue<bool>(true);
        _pos += 4;

      }
      else if(_in[_pos] == 'f' && StringSubstr(_instr, _pos, 5) == "false")
      {

        ret = (JSONValue*)new JSONBaseValue<bool>(false);
        _pos += 5;

      }
      else if(_in[_pos] == 'n' && StringSubstr(_instr, _pos, 4) == "null")
      {

        ret = (JSONValue*)new JSONNull();
        _pos += 4;

      }
      else if(_in[_pos] == 'u' && StringSubstr(_instr, _pos, 9) == "undefined")
      {

        ret = (JSONValue*)new JSONNull();
        _pos += 9;

      }
      else
      {
        if(_in[_pos] != gracefulchar)
        {
          setError(10, "Value: error at position " + (string)_pos);
        }
      }

      if(_errCode != 0 && ret != NULL)
      {
        delete ret;
        ret = NULL;
      }
      return ret;
    }

    JSONArray *parseArray()
    {
      JSONArray *ret = new JSONArray();

      int index = 0;
      skipSpace();
      if(expect('['))
      {
        skipSpace();
        if(peek() != ']')
        {
          while(_errCode == 0)
          {
            // read the value
            JSONValue *v = parseValue(']');
            if(_errCode != 0 || v == NULL) break;

            if(!ret.put(index++, v))
            {
              setError(3, "memory error adding array element " + (string)index);
              break;
            }

            skipSpace();
            if(!expectOptional(',')) break;
            skipSpace();
          }
        }
        if(!expect(']'))
        {
          setError(2, "Array: expected `,` or `]` ");
        }
      }

      if(_errCode != 0)
      {
        delete ret;
        ret = NULL;
      }
      return ret;
    }
    
  public:
    int getErrorCode()
    {
      return _errCode;
    }
    
    string getErrorMessage()
    {
      return _errMsg;
    }
    
    int getPosition()
    {
      return _pos;
    }
    
    /// Parse a sequnce of characters and return a JSONValue.
    JSONValue *parse(string s, int flags = 0) ///< Serialized JSON text
    {
      int inLen;
      JSONValue *ret = NULL;
      StringTrimLeft(s);
      StringTrimRight(s);

      if((flags & JSON_FLAG_REPLACE_BACKTICKS) != 0)
      {
        StringReplace(s,"`","\"");
      }
      else
      if((flags & JSON_FLAG_REPLACE_SINGLETICKS) != 0)
      {
        StringReplace(s,"'","\"");
      }

      _instr = s;
      _len = StringToShortArray(_instr, _in); // nul '0' is added to length
      _pos = 0;
      _errCode = 0;
      _errMsg = "";
      inLen = StringLen(_instr);
      if(_len != inLen + 1 /* nul */)
      {
        setError(1, ("unable to create array " + (string)inLen + " got " + (string)_len));
      }
      else
      {
        _len--;
        ret = parseValue();
        if(_errCode != 0)
        {
          _errMsg = (_errMsg + (_errCode != 10 ? (" at " + (string)_pos) : "") + " [" + StringSubstr(_instr, _pos, 10) + "...]");
        }
      }
      return ret;
    }
    
    static string escape(string a)
    {
    	ushort as[], s[];
    	int n = StringToShortArray(a, as);
    	if(ArrayResize(s, 2 * n) != 2 * n) return NULL;
    	
    	int j = 0;
    	for(int i = 0; i < n; i++)
    	{
    		switch(as[i])
    		{
      		case '\\': s[j]='\\'; j++; s[j]='\\'; j++; break;
      		case '"': s[j]='\\'; j++; s[j]='"'; j++; break;
      		case '/': s[j]='\\'; j++; s[j]='/'; j++; break;
      		case 8: s[j]='\\'; j++; s[j]='b'; j++; break;
      		case 12: s[j]='\\'; j++; s[j]='f'; j++; break;
      		case '\n': s[j]='\\'; j++; s[j]='n'; j++; break;
      		case '\r': s[j]='\\'; j++; s[j]='r'; j++; break;
      		case '\t': s[j]='\\'; j++; s[j]='t'; j++; break;
      		default: s[j]=as[i]; j++; break;
    		}
    	}
    	a = ShortArrayToString(s, 0, j);
    	return a;
    }

};

/// Class to iterate over a JSONObject (not a JSONArray)
class JSONIterator
{
  private:
    ENUM_JSON_TYPE _type;
    HashLoop * _l;
    JSONArray *_a;
    int _cursor;

  public:
    // Create iterator and move to first item
    JSONIterator(JSONObject *jo)
    {
      if(jo == NULL)
      {
        return;
      }
      if(jo.isObject())
      {
        _l = new HashLoop(jo.getHash());
      }
      else
      if(jo.isArray())
      {
        _a = (JSONArray *)jo;
      }
      _type = jo.getType();
      _cursor = 0;
    }
    
    ~JSONIterator()
    {
      if(CheckPointer(_l) == POINTER_DYNAMIC) delete _l;
    }
    
    // Check if more items
    bool hasNext()
    {
      if(_type == JSON_OBJECT)
      {
        if(_l != NULL) return _l.hasNext();
      }
      else
      if(_type == JSON_ARRAY)
      {
        if(_a != NULL) return _cursor < _a.size();
      }
      return false;
    }

    // Move to next item
    void next()
    {
      if(_type == JSON_OBJECT)
      {
        _l.next();
      }
      else
      if(_type == JSON_ARRAY)
      {
        _cursor++;
      }
    }

    // Return item
    JSONValue *val()
    {
      if(_type == JSON_OBJECT)
      {
        return (JSONValue *)(_l.val());
      }
      else
      if(_type == JSON_ARRAY)
      {
        return _a.getValue(_cursor);
      }
      return NULL;
    }

    // Return key
    string key()
    {
      if(_type == JSON_OBJECT)
      {
        return _l.key();
      }
      return NULL;
    }

};

/*
void json_demo_2()
{
  //JSONValue::setFlags(JSON_FLAG_MOVE_OBJECTS);

  string s = "{ \"firstName\": \"John\"," +
             " \"lastName\": \"Smith\"," +
             " \"age\": [25, 35]," +
             " \"address\": { \"streetAddress\": \"21 2nd Street\", \"city\": \"New York\", \"state\": \"NY\", \"postalCode\": \"10021\" }," +
             " \"phoneNumber\": [ { \"type\": \"home\", \"number\": \"212 555-1234\" }, { \"type\": \"fax\", \"number\": \"646 555-4567\" } ]," +
             " \"gender\":{ \"type\":\"male\" }  }";
  Object obj(s);
  
  obj.extend("{\"address\":{\"city\":\"Moscow\"}, \"phoneNumber\": [{\"type\": \"mobile\", \"number\": \"123 456-7890\"}]}");

  Print("firstName:" + obj[_S("firstName")]);
  Print("city:" + obj["address"][_S("city")]);
  obj["address"]["state"] = "Russia"; // change
  Print("state:" + obj["address"][_S("state")]);
  
  //Object *x = 
  obj["gender"] = __O("{\"type\" : \"n/a\"}"); // replace, thanks to JSON_FLAG_MOVE_OBJECTS no need to delete
  //delete x;
  
  //ExampleStruct e = {"empty", 1.0};
  //obj["gender"].extend(e); // extend
  
  //JSONArray *array = (JSONArray *)obj["age"];
  //long age = array[_I(1)];
  //Print("age[2]:" + (string)(((JSONArray *)obj["age"])[_I(1)]));
  Print("age[2]:" + (string)(obj["age"][_L(1)]));
  
  //obj["age"] = _A("[10, 11, 12]");
  int a[] = {10, 11, 12};
  obj["age"].extend(a); // add elements to object-array
  
  //JSONArray *array = (JSONArray *)obj["phoneNumber"];
  //JSONObject *value = (JSONObject *)obj["phoneNumber"][0];

  //Print("phone:" + ((JSONObject *)((JSONArray *)obj["phoneNumber"])[0])[_S("number")]); // value.get<string>
  Print("phone:" + obj["phoneNumber"][0][_S("number")]); // value.get<string>

  // Safe access in case JSON data is missing or different.
  // if (obj.get<string>("firstName", s)) Print("firstName = " + s);

  // Loop over object returning JSONValue
  JSONIterator it(&obj);
  for(; it.hasNext(); it.next())
  {
    Print("loop:" + it.key() + " = " + it.val().toString());
  }
}

void json_demo()
{
  string s = "{ \"firstName\": \"John\"," +
             " \"lastName\": \"Smith\"," +
             " \"age\": 25," +
             " \"address\": { \"streetAddress\": \"21 2nd Street\", \"city\": \"New York\", \"state\": \"NY\", \"postalCode\": \"10021\" }," +
             " \"phoneNumber\": [ { \"type\": \"home\", \"number\": \"212 555-1234\" }, { \"type\": \"fax\", \"number\": \"646 555-4567\" } ]," +
             " \"gender\":{ \"type\":\"male\" }  }";

  JSONParser *parser = new JSONParser();
  JSONValue *jv = parser.parse(s);
  Print("json:");
  if(jv == NULL)
  {
    Print("error:" + (string)parser.getErrorCode() + parser.getErrorMessage());
  }
  else
  {
    Print("PARSED:" + jv.toString());
    if(jv.isObject())
    {
      JSONObject *jo = jv;

      // Direct access - will throw null pointer if wrong getter used.
      Print("firstName:" + jo.get<string>("firstName"));
      Print("city:" + jo.getObject("address").get<string>("city"));
      Print("phone:" + jo.getArray("phoneNumber").getObject(0).get<string>("number"));

      // Safe access in case JSON data is missing or different.
      if(jo.get<string>("firstName", s)) Print("firstName = " + s);

      // Loop over object returning JSONValue
      JSONIterator *it = new JSONIterator(jo);
      for(; it.hasNext() ; it.next())
      {
        Print("loop:" + it.key() + " = " + it.val().toString());
      }
      delete it;
    }
    delete jv;
  }
  delete parser;
}
*/
static int JSONValue::_flags = 0;
static int JSONValue::_digits = 8;

class JSONValueInit: public JSONArray
{
  public:
    JSONValueInit()
    {
      //initializer<int>(0);
      initializer<string>("");
      initializer<bool>(false);
      initializer<long>(0);
      initializer<double>(0.0);

      //initializer2<int>(0);
      initializer2<string>("");
      initializer2<bool>(false);
      initializer2<long>(0);
      initializer2<double>(0.0);
      
      //initializer3<int>(0);
      initializer3<string>("");
      initializer3<bool>(false);
      initializer3<long>(0);
      initializer3<double>(0.0);
      
      JSONBaseValue<bool> b;
      //JSONBaseValue<int> i;
      JSONBaseValue<long> l;
      JSONBaseValue<double> d;
      JSONBaseValue<string> s;
      
      JSONObject x;
      JSONObject obj;
      JSONArray arr;
      JSONNull nul;
      x = &obj;
      x = &arr;
      x = &nul;
      x = &b;
      x = &l;
      x = &d;
      x = &s;
      JSONArray::filterByArray<bool>(NULL ,NULL ,NULL);
      JSONArray::filterByArray<double>(NULL ,NULL ,NULL);
      JSONArray::filterByArray<long>(NULL ,NULL ,NULL);
      JSONArray::filterByArray<string>(NULL ,NULL ,NULL);
      //JSONArray::filterByArray<int>(NULL ,NULL ,NULL);
    }
};
JSONValueInit _jsvi_;
